type __ = Obj.t
let __ = let rec f _ = Obj.repr f in Obj.repr f

type bool =
  | True
  | False

type nat =
  | O
  | S of nat

type 'a option =
  | Some of 'a
  | None

type comparison =
  | Eq
  | Lt
  | Gt

(** val compOpp : comparison -> comparison **)

let compOpp = function
  | Eq -> Eq
  | Lt -> Gt
  | Gt -> Lt

type 'a sig0 = 'a
  (* singleton inductive, whose constructor was exist *)

type sumbool =
  | Left
  | Right

type 'a sumor =
  | Inleft of 'a
  | Inright

type positive =
  | XI of positive
  | XO of positive
  | XH

(** val psucc : positive -> positive **)

let rec psucc = function
  | XI x' -> XO (psucc x')
  | XO x' -> XI x'
  | XH -> XO XH

(** val pplus : positive -> positive -> positive **)

let rec pplus x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> XO (pplus_carry x' y')
           | XO y' -> XI (pplus x' y')
           | XH -> XO (psucc x'))
    | XO x' ->
        (match y with
           | XI y' -> XI (pplus x' y')
           | XO y' -> XO (pplus x' y')
           | XH -> XI x')
    | XH ->
        (match y with
           | XI y' -> XO (psucc y')
           | XO y' -> XI y'
           | XH -> XO XH)

(** val pplus_carry : positive -> positive -> positive **)

and pplus_carry x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> XI (pplus_carry x' y')
           | XO y' -> XO (pplus_carry x' y')
           | XH -> XI (psucc x'))
    | XO x' ->
        (match y with
           | XI y' -> XO (pplus_carry x' y')
           | XO y' -> XI (pplus x' y')
           | XH -> XO (psucc x'))
    | XH ->
        (match y with
           | XI y' -> XI (psucc y')
           | XO y' -> XO (psucc y')
           | XH -> XI XH)

(** val pdouble_minus_one : positive -> positive **)

let rec pdouble_minus_one = function
  | XI x' -> XI (XO x')
  | XO x' -> XI (pdouble_minus_one x')
  | XH -> XH

type positive_mask =
  | IsNul
  | IsPos of positive
  | IsNeg

(** val pdouble_plus_one_mask : positive_mask -> positive_mask **)

let pdouble_plus_one_mask = function
  | IsNul -> IsPos XH
  | IsPos p -> IsPos (XI p)
  | IsNeg -> IsNeg

(** val pdouble_mask : positive_mask -> positive_mask **)

let pdouble_mask = function
  | IsNul -> IsNul
  | IsPos p -> IsPos (XO p)
  | IsNeg -> IsNeg

(** val pdouble_minus_two : positive -> positive_mask **)

let pdouble_minus_two = function
  | XI x' -> IsPos (XO (XO x'))
  | XO x' -> IsPos (XO (pdouble_minus_one x'))
  | XH -> IsNul

(** val pminus_mask : positive -> positive -> positive_mask **)

let rec pminus_mask x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pdouble_mask (pminus_mask x' y')
           | XO y' -> pdouble_plus_one_mask (pminus_mask x' y')
           | XH -> IsPos (XO x'))
    | XO x' ->
        (match y with
           | XI y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_mask (pminus_mask x' y')
           | XH -> IsPos (pdouble_minus_one x'))
    | XH -> (match y with
               | XI p -> IsNeg
               | XO p -> IsNeg
               | XH -> IsNul)

(** val pminus_mask_carry : positive -> positive -> positive_mask **)

and pminus_mask_carry x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_mask (pminus_mask x' y')
           | XH -> IsPos (pdouble_minus_one x'))
    | XO x' ->
        (match y with
           | XI y' -> pdouble_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XH -> pdouble_minus_two x')
    | XH -> IsNeg

(** val pminus : positive -> positive -> positive **)

let pminus x y =
  match pminus_mask x y with
    | IsNul -> XH
    | IsPos z0 -> z0
    | IsNeg -> XH

(** val pcompare : positive -> positive -> comparison -> comparison **)

let rec pcompare x y r =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pcompare x' y' r
           | XO y' -> pcompare x' y' Gt
           | XH -> Gt)
    | XO x' ->
        (match y with
           | XI y' -> pcompare x' y' Lt
           | XO y' -> pcompare x' y' r
           | XH -> Gt)
    | XH -> (match y with
               | XI y' -> Lt
               | XO y' -> Lt
               | XH -> r)

type z =
  | Z0
  | Zpos of positive
  | Zneg of positive

(** val zplus : z -> z -> z **)

let zplus x y =
  match x with
    | Z0 -> y
    | Zpos x' ->
        (match y with
           | Z0 -> Zpos x'
           | Zpos y' -> Zpos (pplus x' y')
           | Zneg y' ->
               (match pcompare x' y' Eq with
                  | Eq -> Z0
                  | Lt -> Zneg (pminus y' x')
                  | Gt -> Zpos (pminus x' y')))
    | Zneg x' ->
        (match y with
           | Z0 -> Zneg x'
           | Zpos y' ->
               (match pcompare x' y' Eq with
                  | Eq -> Z0
                  | Lt -> Zpos (pminus y' x')
                  | Gt -> Zneg (pminus x' y'))
           | Zneg y' -> Zneg (pplus x' y'))

(** val zcompare : z -> z -> comparison **)

let zcompare x y =
  match x with
    | Z0 -> (match y with
               | Z0 -> Eq
               | Zpos y' -> Lt
               | Zneg y' -> Gt)
    | Zpos x' ->
        (match y with
           | Z0 -> Gt
           | Zpos y' -> pcompare x' y' Eq
           | Zneg y' -> Gt)
    | Zneg x' ->
        (match y with
           | Z0 -> Lt
           | Zpos y' -> Lt
           | Zneg y' -> compOpp (pcompare x' y' Eq))

(** val dcompare_inf : comparison -> sumbool sumor **)

let dcompare_inf = function
  | Eq -> Inleft Left
  | Lt -> Inleft Right
  | Gt -> Inright

(** val zcompare_rec : z -> z -> (__ -> 'a1) -> (__ -> 'a1) -> (__ -> 'a1) ->
                       'a1 **)

let zcompare_rec x y h1 h2 h3 =
  match dcompare_inf (zcompare x y) with
    | Inleft x0 -> (match x0 with
                      | Left -> h1 __
                      | Right -> h2 __)
    | Inright -> h3 __

(** val z_le_dec : z -> z -> sumbool **)

let z_le_dec x y =
  zcompare_rec x y (fun _ -> Left) (fun _ -> Left) (fun _ -> Right)

(** val z_le_gt_dec : z -> z -> sumbool **)

let z_le_gt_dec x y =
  z_le_dec x y

type 'a list =
  | Nil
  | Cons of 'a * 'a list

type ('var, 'aExp, 'bExp) inst =
  | Skip
  | Assign of 'var * 'aExp
  | Scolon of ('var, 'aExp, 'bExp) inst * ('var, 'aExp, 'bExp) inst
  | WhileDo of 'bExp * ('var, 'aExp, 'bExp) inst

type ('var, 'aExp, 'bExp) inst_cases_type =
  | Is_Skip
  | Is_assign of 'var * 'aExp
  | Is_seq of ('var, 'aExp, 'bExp) inst * ('var, 'aExp, 'bExp) inst
  | Is_while of 'bExp * ('var, 'aExp, 'bExp) inst

(** val inst_cases : ('a1, 'a2, 'a3) inst -> ('a1, 'a2, 'a3) inst_cases_type **)

let inst_cases = function
  | Skip -> Is_Skip
  | Assign (v, e) -> Is_assign (v, e)
  | Scolon (i1, i2) -> Is_seq (i1, i2)
  | WhileDo (b, i0) -> Is_while (b, i0)

type ('var, 'aExp, 'bExp, 'state) exec_or_not =
  | Result of 'state
  | No_result

(** val exec_assign : ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option)
                      -> ('a4 -> 'a1 -> z -> 'a4 option) -> ('a1, 'a2, 'a3)
                      inst -> 'a4 -> 'a1 -> 'a2 -> ('a1, 'a2, 'a3, 'a4)
                      exec_or_not **)

let exec_assign evalA0 evalB0 update0 i s v e =
  match evalA0 s e with
    | Some v' ->
        (match update0 s v v' with
           | Some s' -> Result s'
           | None -> No_result)
    | None -> No_result

(** val exec_sequence : ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool
                        option) -> ('a4 -> 'a1 -> z -> 'a4 option) -> ('a1,
                        'a2, 'a3) inst -> 'a4 -> ('a1, 'a2, 'a3) inst ->
                        ('a1, 'a2, 'a3) inst -> ('a4 -> ('a1, 'a2, 'a3, 'a4)
                        exec_or_not) -> ('a4 -> ('a1, 'a2, 'a3, 'a4)
                        exec_or_not) -> ('a1, 'a2, 'a3, 'a4) exec_or_not **)

let exec_sequence evalA0 evalB0 update0 i s i1 i2 f1 f2 =
  match f1 s with
    | Result s1 ->
        (match f2 s1 with
           | Result s2 -> Result s2
           | No_result -> No_result)
    | No_result -> No_result

(** val exec_forLoop : ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool
                       option) -> ('a4 -> 'a1 -> z -> 'a4 option) -> ('a1,
                       'a2, 'a3) inst -> ('a1, 'a2, 'a3) inst -> 'a3 -> 'a4
                       -> ('a4 -> ('a1, 'a2, 'a3, 'a4) exec_or_not) -> ('a1,
                       'a2, 'a3, 'a4) exec_or_not **)

let rec exec_forLoop evalA0 evalB0 update0 i i' b s f =
  match evalB0 s b with
    | Some vb ->
        (match vb with
           | True ->
               (match f s with
                  | Result x ->
                      (match exec_forLoop evalA0 evalB0 update0 i i' b x f with
                         | Result x0 -> Result x0
                         | No_result -> No_result)
                  | No_result -> No_result)
           | False -> Result s)
    | None -> No_result

(** val exl : ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4
              -> 'a1 -> z -> 'a4 option) -> ('a1, 'a2, 'a3) inst -> 'a4 ->
              ('a1, 'a2, 'a3, 'a4) exec_or_not **)

let rec exl evalA0 evalB0 update0 i s =
  match inst_cases i with
    | Is_Skip -> Result s
    | Is_assign (v, e) -> exec_assign evalA0 evalB0 update0 i s v e
    | Is_seq (i1, i2) ->
        exec_sequence evalA0 evalB0 update0 i s i1 i2 (fun s0 ->
          exl evalA0 evalB0 update0 i1 s0) (fun s0 ->
          exl evalA0 evalB0 update0 i2 s0)
    | Is_while (b, i') ->
        exec_forLoop evalA0 evalB0 update0 i i' b s (fun s0 ->
          exl evalA0 evalB0 update0 i' s0)

(** val execute_loops : ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool
                        option) -> ('a4 -> 'a1 -> z -> 'a4 option) -> 'a4 ->
                        ('a1, 'a2, 'a3) inst -> 'a4 sumor **)

let execute_loops evalA0 evalB0 update0 s i =
  match exl evalA0 evalB0 update0 i s with
    | Result x -> Inleft x
    | No_result -> Inright

type aExp =
  | APlus of aExp * aExp
  | ANum of z
  | AVar of nat

type bExp =
  | BGt of aExp * aExp

type state = z list

(** val lookup : nat -> state -> z option **)

let rec lookup idx s =
  match idx with
    | O -> (match s with
              | Nil -> None
              | Cons (a, l) -> Some a)
    | S p -> (match s with
                | Nil -> None
                | Cons (z0, tl) -> lookup p tl)

(** val evalA : state -> aExp -> z option **)

let rec evalA s = function
  | APlus (e1, e2) ->
      (match evalA s e1 with
         | Some n1 ->
             (match evalA s e2 with
                | Some n2 -> Some (zplus n1 n2)
                | None -> None)
         | None -> None)
  | ANum n -> Some n
  | AVar n -> lookup n s

(** val evalB : state -> bExp -> bool option **)

let evalB s = function
  | BGt (e1, e2) ->
      (match evalA s e1 with
         | Some n1 ->
             (match evalA s e2 with
                | Some n2 ->
                    (match z_le_gt_dec n1 n2 with
                       | Left -> Some False
                       | Right -> Some True)
                | None -> None)
         | None -> None)

(** val update : state -> nat -> z -> state option **)

let rec update s n x =
  match s with
    | Nil -> None
    | Cons (a, tl) ->
        (match n with
           | O -> Some (Cons (x, tl))
           | S p ->
               (match update tl p x with
                  | Some tl' -> Some (Cons (a, tl'))
                  | None -> None))

(** val fact3 : (nat, aExp, bExp) inst **)

let fact3 =
  Scolon ((Assign ((S O), (ANum (Zpos (XI XH))))), (Scolon ((Assign ((S (S
    O)), (ANum (Zpos XH)))), (WhileDo ((BGt ((AVar (S O)), (ANum Z0))),
    (Scolon ((Assign (O, (AVar (S O)))), (Scolon ((Assign ((S O), (APlus
    ((AVar (S O)), (ANum (Zneg XH)))))), (Scolon ((Assign ((S (S (S O))),
    (AVar (S (S O))))), (Scolon ((Assign ((S (S O)), (ANum Z0))), (WhileDo
    ((BGt ((AVar O), (ANum Z0))), (Scolon ((Assign (O, (APlus ((AVar O),
    (ANum (Zneg XH)))))), (Assign ((S (S O)), (APlus ((AVar (S (S O))), (AVar
    (S (S (S O)))))))))))))))))))))))))

(** val execute_loops2 : state -> (nat, aExp, bExp) inst -> state sumor **)

let execute_loops2 s i =
  execute_loops evalA evalB update s i

(** val value : state sumor **)

let value =
  execute_loops2 (Cons (Z0, (Cons (Z0, (Cons (Z0, (Cons (Z0, Nil))))))))
    fact3

