type __ = Obj.t

type bool =
  | True
  | False

type nat =
  | O
  | S of nat

type 'a option =
  | Some of 'a
  | None

type comparison =
  | Eq
  | Lt
  | Gt

val compOpp : comparison -> comparison

type 'a sig0 = 'a
  (* singleton inductive, whose constructor was exist *)

type sumbool =
  | Left
  | Right

type 'a sumor =
  | Inleft of 'a
  | Inright

type positive =
  | XI of positive
  | XO of positive
  | XH

val psucc : positive -> positive

val pplus : positive -> positive -> positive

val pplus_carry : positive -> positive -> positive

val pdouble_minus_one : positive -> positive

type positive_mask =
  | IsNul
  | IsPos of positive
  | IsNeg

val pdouble_plus_one_mask : positive_mask -> positive_mask

val pdouble_mask : positive_mask -> positive_mask

val pdouble_minus_two : positive -> positive_mask

val pminus_mask : positive -> positive -> positive_mask

val pminus_mask_carry : positive -> positive -> positive_mask

val pminus : positive -> positive -> positive

val pcompare : positive -> positive -> comparison -> comparison

type z =
  | Z0
  | Zpos of positive
  | Zneg of positive

val zplus : z -> z -> z

val zcompare : z -> z -> comparison

val dcompare_inf : comparison -> sumbool sumor

val zcompare_rec : z -> z -> (__ -> 'a1) -> (__ -> 'a1) -> (__ -> 'a1) -> 'a1

val z_le_dec : z -> z -> sumbool

val z_le_gt_dec : z -> z -> sumbool

type 'a list =
  | Nil
  | Cons of 'a * 'a list

type ('var, 'aExp, 'bExp) inst =
  | Skip
  | Assign of 'var * 'aExp
  | Scolon of ('var, 'aExp, 'bExp) inst * ('var, 'aExp, 'bExp) inst
  | WhileDo of 'bExp * ('var, 'aExp, 'bExp) inst

type ('var, 'aExp, 'bExp) inst_cases_type =
  | Is_Skip
  | Is_assign of 'var * 'aExp
  | Is_seq of ('var, 'aExp, 'bExp) inst * ('var, 'aExp, 'bExp) inst
  | Is_while of 'bExp * ('var, 'aExp, 'bExp) inst

val inst_cases : ('a1, 'a2, 'a3) inst -> ('a1, 'a2, 'a3) inst_cases_type

type ('var, 'aExp, 'bExp, 'state) exec_or_not =
  | Result of 'state
  | No_result

val exec_assign :
  ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4 -> 'a1 -> z
  -> 'a4 option) -> ('a1, 'a2, 'a3) inst -> 'a4 -> 'a1 -> 'a2 -> ('a1, 'a2,
  'a3, 'a4) exec_or_not

val exec_sequence :
  ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4 -> 'a1 -> z
  -> 'a4 option) -> ('a1, 'a2, 'a3) inst -> 'a4 -> ('a1, 'a2, 'a3) inst ->
  ('a1, 'a2, 'a3) inst -> ('a4 -> ('a1, 'a2, 'a3, 'a4) exec_or_not) -> ('a4
  -> ('a1, 'a2, 'a3, 'a4) exec_or_not) -> ('a1, 'a2, 'a3, 'a4) exec_or_not

val exec_forLoop :
  ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4 -> 'a1 -> z
  -> 'a4 option) -> ('a1, 'a2, 'a3) inst -> ('a1, 'a2, 'a3) inst -> 'a3 ->
  'a4 -> ('a4 -> ('a1, 'a2, 'a3, 'a4) exec_or_not) -> ('a1, 'a2, 'a3, 'a4)
  exec_or_not

val exl :
  ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4 -> 'a1 -> z
  -> 'a4 option) -> ('a1, 'a2, 'a3) inst -> 'a4 -> ('a1, 'a2, 'a3, 'a4)
  exec_or_not

val execute_loops :
  ('a4 -> 'a2 -> z option) -> ('a4 -> 'a3 -> bool option) -> ('a4 -> 'a1 -> z
  -> 'a4 option) -> 'a4 -> ('a1, 'a2, 'a3) inst -> 'a4 sumor

type aExp =
  | APlus of aExp * aExp
  | ANum of z
  | AVar of nat

type bExp =
  | BGt of aExp * aExp

type state = z list

val lookup : nat -> state -> z option

val evalA : state -> aExp -> z option

val evalB : state -> bExp -> bool option

val update : state -> nat -> z -> state option

val fact3 : (nat, aExp, bExp) inst

val execute_loops2 : state -> (nat, aExp, bExp) inst -> state sumor

val value : state sumor

